"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
var tslib_1 = require("tslib");
var Configuration_1 = tslib_1.__importDefault(require("./Configuration"));
var FileUtils_1 = tslib_1.__importDefault(require("./FileUtils"));
var moment_1 = tslib_1.__importDefault(require("moment"));
var utils_1 = require("./utils");
var formatFileName = function (fileName) { return fileName.replace(' ', '-'); };
var LogTypes;
(function (LogTypes) {
    LogTypes["INFO"] = "info";
    LogTypes["WARNING"] = "warning";
    LogTypes["ERROR"] = "error";
})(LogTypes || (LogTypes = {}));
var Logger = /** @class */ (function () {
    function Logger(jobInfo, loggerConfig) {
        var _a, _b;
        var _this = this;
        this.logs = (_a = {},
            _a[LogTypes.INFO] = [],
            _a[LogTypes.WARNING] = [],
            _a[LogTypes.ERROR] = [],
            _a);
        this.hoistedLogs = (_b = {},
            _b[LogTypes.INFO] = [],
            _b[LogTypes.WARNING] = [],
            _b[LogTypes.ERROR] = [],
            _b);
        this.loggerConfig = {
            debugLevel: 3,
            removeLocalLogFileOnSave: false,
        };
        this.initialSaveDone = false;
        this.useAdapter = function (adapter) {
            _this.adapter = adapter;
        };
        // Use these methods to log the execution time of the task
        this.startLogTime = function () {
            _this.startTime = moment_1.default(new Date());
        };
        this.endLogTime = function () {
            _this.endTime = moment_1.default(new Date());
        };
        // Utility methods to log messages or errors at different levels
        this.info = function (message, config) {
            if (_this.loggerConfig.debugLevel < 3) {
                return console.log('Debug level is less than 3, [INFO] messaged will not be logged');
            }
            _this.storeLog(message, LogTypes.INFO, config);
        };
        this.warn = function (message, config) {
            if (_this.loggerConfig.debugLevel < 3) {
                return console.log('Debug level is less than 2, [WARNING] messaged will not be logged');
            }
            _this.storeLog(message, LogTypes.WARNING, config);
        };
        this.error = function (message, config) {
            if (_this.loggerConfig.debugLevel < 1) {
                return console.log('Debug level is less than 1, No messages will be logged');
            }
            _this.storeLog(message, LogTypes.ERROR, config);
        };
        this.storeLog = function (logMessage, logType, logConfig) {
            var formattedMessage = _this.formatMessageByTag(logMessage, "[" + logType.toUpperCase() + "]");
            if (logConfig === null || logConfig === void 0 ? void 0 : logConfig.hoisted) {
                _this.hoistedLogs[logType].push(formattedMessage);
            }
            else {
                _this.logs[logType].push(formattedMessage);
            }
        };
        this.setJobStatus = function (statusCode) {
            _this.jobInfo.set('STATUS_CODE', statusCode);
        };
        this.getTaskDuration = function () { return moment_1.default.duration(_this.endTime.diff(_this.startTime)); };
        this.getNumberOfLogsByType = function (logType) {
            return _this.logs[logType].length + _this.hoistedLogs[logType].length;
        };
        this.save = function (_a) {
            var _b = _a === void 0 ? {
                saveAsNew: false,
                clearLogsOnSave: false,
            } : _a, saveAsNew = _b.saveAsNew, clearLogsOnSave = _b.clearLogsOnSave;
            return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var logFiles;
                return tslib_1.__generator(this, function (_c) {
                    switch (_c.label) {
                        case 0:
                            logFiles = saveAsNew ? this.setUpNewLogFileHandlers() : this.logFiles;
                            if (!saveAsNew) {
                                this.clearLogFiles();
                            }
                            this.writeJobInfo(logFiles);
                            this.writeLogs(logFiles);
                            if (!this.adapter) return [3 /*break*/, 2];
                            return [4 /*yield*/, this.writeToAdapter()];
                        case 1:
                            _c.sent();
                            _c.label = 2;
                        case 2:
                            if (clearLogsOnSave)
                                this.clearLogs();
                            if (this.loggerConfig.removeLocalLogFileOnSave)
                                Object.values(logFiles).forEach(function (logFile) { return logFile.delete(); });
                            return [2 /*return*/];
                    }
                });
            });
        };
        this.clearLogFiles = function () {
            Object.values(_this.logFiles).forEach(function (logFile) { return logFile.clearFile(); });
        };
        this.clearLogs = function () {
            var _a, _b;
            _this.logs = (_a = {},
                _a[LogTypes.INFO] = [],
                _a[LogTypes.WARNING] = [],
                _a[LogTypes.ERROR] = [],
                _a);
            _this.hoistedLogs = (_b = {},
                _b[LogTypes.INFO] = [],
                _b[LogTypes.WARNING] = [],
                _b[LogTypes.ERROR] = [],
                _b);
        };
        this.writeToAdapter = function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var jobTag;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        jobTag = this.jobInfo.get('name') + "-" + this.startTime.format('DD-MM-YYYY | hh:mm:ss');
                        return [4 /*yield*/, Promise.all(Object.entries(this.logFiles).map(function (_a) {
                                var logType = _a[0], logFile = _a[1];
                                if (logFile.isAvailableOnDisk)
                                    return _this.adapter.save(logFile.filePath, logFile.filename, jobTag + "/" + logType);
                            }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); };
        this.writeLogs = function (logFiles) {
            Object.entries(_this.hoistedLogs).forEach(function (_a) {
                var logType = _a[0], logs = _a[1];
                var logFile = logFiles[logType];
                if (logs.length > 0) {
                    logFile.write('\n===========================================\n');
                    logFile.write('HOISTED LOGS\n');
                    logFile.write('===========================================\n');
                    logs.forEach(function (log) {
                        logFile.write(log);
                        logFile.write('\n');
                        logFile.write('\n');
                    });
                }
            });
            Object.entries(_this.logs).forEach(function (_a) {
                var logType = _a[0], logs = _a[1];
                if (logs.length > 0) {
                    var logFile_1 = logFiles[logType];
                    logFile_1.write('\n===========================================\n');
                    logFile_1.write('LOGS\n');
                    logFile_1.write('===========================================\n');
                    logs.forEach(function (log) {
                        logFile_1.write(log);
                        logFile_1.write('\n');
                        logFile_1.write('\n');
                    });
                }
            });
        };
        this.writeJobInfo = function (logFiles) {
            Object.entries(logFiles).forEach(function (_a) {
                var logType = _a[0], logFile = _a[1];
                if (_this.getNumberOfLogsByType(logType) === 0)
                    return;
                logFile.write('===========================================\n');
                logFile.write('JOB INFORMATION\n');
                logFile.write('===========================================\n');
                var jobConfig = _this.jobInfo.getAll();
                Object.keys(jobConfig).forEach(function (key) {
                    var logMessage = utils_1.toString(jobConfig[key]);
                    logFile.write(key.toUpperCase() + " : " + logMessage + "\n");
                });
                logFile.write("STARTED AT : " + _this.startTime.format('DD-MM-YYYY | hh:mm:ss') + "\n");
                logFile.write("FINISHED AT : " + _this.endTime.format('DD-MM-YYYY | hh:mm:ss') + "\n");
                var taskDuration = _this.getTaskDuration();
                logFile.write("EXECUTION TIME : " + taskDuration.hours() + " hrs " + taskDuration.minutes() + " min " + taskDuration.seconds() + " sec " + taskDuration.milliseconds() + " ms\n");
            });
        };
        if (jobInfo) {
            this.jobInfo = new Configuration_1.default(jobInfo);
        }
        if (loggerConfig)
            this.loggerConfig = loggerConfig
                ? tslib_1.__assign(tslib_1.__assign({}, this.loggerConfig), loggerConfig) : this.loggerConfig;
        this.logFiles = this.setUpNewLogFileHandlers();
    }
    Logger.prototype.updateConfig = function (jobInfo, loggerConfig) {
        this.jobInfo = new Configuration_1.default(jobInfo);
        this.loggerConfig = loggerConfig
            ? tslib_1.__assign(tslib_1.__assign({}, this.loggerConfig), loggerConfig) : this.loggerConfig;
    };
    Logger.prototype.setUpNewLogFileHandlers = function () {
        var _a;
        var jobName = formatFileName(this.jobInfo.get('name'));
        var jobDate = moment_1.default(new Date()).format('DD-MM-YYYY-hh-mm-x');
        return _a = {},
            _a[LogTypes.INFO] = new FileUtils_1.default(jobName + "-INFO-" + jobDate + ".log"),
            _a[LogTypes.WARNING] = new FileUtils_1.default(jobName + "-WARNINGS-" + jobDate + ".log"),
            _a[LogTypes.ERROR] = new FileUtils_1.default(jobName + "-ERRORS-" + jobDate + ".log"),
            _a;
    };
    Logger.prototype.formatMessageByTag = function (message, tag) {
        var isString = typeof message === 'string';
        message = utils_1.toString(message);
        var spacer = isString ? '\t' : '\n';
        return tag + " | " + moment_1.default().format('hh:mm:ss') + " " + spacer + message;
    };
    return Logger;
}());
exports.Logger = Logger;
exports.default = Logger;
//# sourceMappingURL=Logger.js.map